@@ -62,10 +62,11 @@ class StatsigServer:
             globals.logger.info("Statsig is already initialized.")
             return InitializeDetails(0, self.get_init_source(), True, self.is_store_populated(), None)
 
-        if sdkKey is None or not sdkKey.startswith("secret-"):
-            raise StatsigValueError(
+        if sdkKey is None or (not sdkKey.startswith("secret-") and not options.local_mode):
+            globals.logger.info(
                 "Invalid key provided. You must use a Server Secret Key from the Statsig console."
             )
+            return InitializeDetails(0, DataSource.UNINITIALIZED, False, False, None)
 
         environment_tier = options.get_sdk_environment_tier() or 'unknown'
         globals.logger.info(
@@ -123,7 +124,7 @@ class StatsigServer:
                 init_context
             )
 
-            self._evaluator = _Evaluator(self._spec_store)
+            self._evaluator = _Evaluator(self._spec_store, self._options.global_custom_fields, self._options.disable_ua_parser, self._options.disable_country_lookup)
 
             init_timeout = options.overall_init_timeout
             if init_timeout is not None:
@@ -186,6 +187,7 @@ class StatsigServer:
                     False,
                     gate_name,
                     "",
+                    "",
                 )
                 if not self._options.evaluation_callback is None:
                     self._options.evaluation_callback(feature_gate)
@@ -196,8 +198,10 @@ class StatsigServer:
                 result.boolean_value,
                 gate_name,
                 result.rule_id,
+                result.id_type,
                 result.group_name,
-                result.evaluation_details
+                result.evaluation_details,
+                result.override_config_name
             )
             self.safe_eval_callback(feature_gate)
             return feature_gate
@@ -315,7 +319,7 @@ class StatsigServer:
 
     def manually_log_experiment_exposure(self, user: StatsigUser, experiment_name: str):
         user = self.__normalize_user(user)
-        result = self._evaluator.get_config(user, experiment_name, True)
+        result = self._evaluator.get_config(user, experiment_name)
         self._logger.log_config_exposure(
             user,
             experiment_name,
